#!/usr/bin/env python
# -*- coding:utf-8 _*-
"""
@author: xl
@file: conf.py
@time: 2021/08/09
@contact:
@site:
@software: PyCharm
"""
import os

import attrdict
from dotenv import load_dotenv
from loguru import logger

load_dotenv(verbose=True)

METRICS_AGENT_HOST = os.environ.get('METRICS_AGENT_HOST', 'http://127.0.0.1:10039')
METRICS_APP_NAME = os.environ.get('METRICS_APP_NAME', os.environ.get('MY_PRODUCT_NAME', 'service-monitor'))
METRICS_SERVICE_NAME = os.environ.get('METRICS_SERVICE_NAME', os.environ.get('MY_PROJECT_NAME', 'service-monitor-test'))
METRICS_APP_VERSION = os.environ.get('METRICS_APP_VERSION', os.environ.get('MY_VERSION_NAME', 'test'))
METRICS_SKIP_1ST_PERIOD = os.environ.get('METRICS_SKIP_1ST_PERIOD', 'False')
METRICS_STEP = os.environ.get('METRICS_STEP', '60')

CONF = attrdict.AttrDict({
    "METRICS_AGENT_HOST": METRICS_AGENT_HOST,
    "METRICS_APP_NAME": METRICS_APP_NAME,
    'METRICS_SERVICE_NAME': METRICS_SERVICE_NAME,
    'METRICS_APP_VERSION': METRICS_APP_VERSION,
    'METRICS_SKIP_1ST_PERIOD': METRICS_SKIP_1ST_PERIOD.lower() == 'true',
    'METRICS_STEP': int(METRICS_STEP)
})

logger.info(CONF)



import time
import traceback
from typing import List

from loguru import logger

from rosetta_service_monitor.client.service.metrics_service import metrics_service
from fastapi import Response


def register_hook(app, project_name, scale: List[int] = None, **kwargs) -> None:
    """
    请求响应拦截fastapi请求上下文进行hook
    https://fastapi.tiangolo.com/tutorial/middleware/
    :param project_name:
    :param app:
    :param scale: stat scale.
    :return:
    """
    if not scale:
        scale = [i * 100 * 1000 for i in range(1, 21, 1)]
    @app.middleware("http")
    async def monitor(request, call_next):
        path = request.url.path.lstrip('/')
        response = None
        try:
            start = time.time()
            response: Response = await call_next(request)
        except KeyboardInterrupt:
            raise KeyboardInterrupt
        except Exception as err:
            info = f'{err}:{err.__class__.__name__}\n{traceback.format_exc()}'
            logger.error(info)
            defmodel = {
                "topic": "",
                "uri": path,
                "uri_tag": "s",
                "duration": int((time.time() - start) * 1e6),
                "code": response.status_code if response else 500,
                "isSuccess": "n",
                "scale": scale
            }
            metrics_service.send_async(defmodel=defmodel)
            raise err

        defmodel = {
            "topic": "",
            "uri": path,
            "uri_tag": "s",
            "duration": int((time.time() - start) * 1e6),
            "code": response.status_code,
            "isSuccess": "y",
            "scale": scale
        }

        metrics_service.send_async(defmodel=[defmodel])

        return response



#!/usr/bin/env python  
# -*- coding:utf-8 _*-
""" 
@author: xl
@file: __init__.py.py 
@time: 2021/08/09
@contact: 
@site:  
@software: PyCharm 
"""
import abc
import os
import socket
import time

from loguru import logger

from rosetta_service_monitor.client.enums import WarnningType
from rosetta_service_monitor.tasks.baskground_task_actor import BackgroundWorker


class MQBaseService(metaclass=abc.ABCMeta):
    def __init__(self):
        # 启动一个后台线程，用于执行后台任务
        self._worker = BackgroundWorker()

    @abc.abstractmethod
    async def send_mq(self, project_name: str, err_msg: str, trace_id: str, request: dict,
                      warnning_type: WarnningType = WarnningType.normal, **kwargs):
        pass

    def async_run(self, func, *args, **kwargs):
        """
        发起后台任务
        :param func:
        :param args:
        :param kwargs:
        :return: f 通过f.result() 获取对应执行结果
        """
        f = self._worker.submit(func, *args, **kwargs)
        return f

    def close(self):
        ...


# 用于本地连不上mq，异步测试
class MQNoUseService(MQBaseService):
    def __init__(self):
        super(MQNoUseService, self).__init__()

    async def send_mq(self, project_name: str, err_msg: str, trace_id: str, request: dict,
                      warnning_type: WarnningType = WarnningType.normal, **kwargs):
        """
        发送消息到mq,
        :param project_name:
        :param err_msg:
        :param trace_id:
        :param request:
        :param warnning_type:
        :param kwargs:
        :return: None
        """
        hostname = socket.gethostname()
        time.sleep(5)
        host_info = {
            'hostname': hostname,
            'ip': os.getenv("HOST_IP"),
            'pod_name': os.getenv("MY_POD_NAME"),
            'pod_ip': os.getenv("POD_IP")
        }

        msg = {
            'project_name': project_name,
            'trace_id': trace_id,
            'reason': err_msg,
            'request': request,
            'hostinfo': host_info,
            'stage': warnning_type.value
        }
        msg.update(kwargs)
        logger.info(f"send msg: {msg}")
        return None

    def send_msg_async(self, project_name: str, err_msg: str, trace_id: str, request: dict,
                       warnning_type: WarnningType = WarnningType.normal, **kwargs):
        r = self.async_run(self.send_mq, project_name, err_msg, trace_id, request,
                           warnning_type, **kwargs)
        return r
